/* -------------------------------------------------------------------------
 * Perto S/A
 * File:    testpind.c
 * Purpose: Testing functions to "talk to" a Verifone SC552 PINPAD.
 * Programmer: Tarcisio Kreutz
 * Date:    2003/04/15
 * ------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <termio.h>
#include <errno.h>

#include <perto.h>

#include "testpind.h"

#define T_STX 2
#define T_ETX 3
#define T_EOT 4
#define T_ACK 6
#define T_SO  14
#define T_SI  15
#define T_NAK 21
#define T_FS  28

/* Return a device handle if OK, or (-1) if error */
#ifdef ANSI
int pin_open_dev(char *dev, int baud)
#else
int pin_open_dev(dev, baud)
	char *dev; int baud;
#endif /* ANSI */
{
   struct termio tbuf;
	int handle;

	baud = (baud == 1200 ? B1200 : 
				baud == 2400 ? B2400 : 
					baud == 4800 ? B4800 : 
						baud == 9600 ? B9600 : B1200);

   if ((handle = open(dev, O_RDWR)) == -1)
   {
      fprintf(stderr, "ERRO: Falha abertura porta serial. [%s: %s]\n",
              dev, strerror(errno));
      return -1;
   }

   memset((char *)&tbuf, 0, sizeof(tbuf));

   tbuf.c_iflag = IGNBRK;
   tbuf.c_oflag = 0;
   tbuf.c_cflag = baud | CS8 | CREAD | CLOCAL | HUPCL;
   tbuf.c_lflag = 0;
   tbuf.c_cc[VMIN] = 0;
   tbuf.c_cc[VTIME] = 2;        /* 2 * 0.1s */

   if (ioctl(handle, TCSETAF, &tbuf) == -1)
   {
      fprintf(stderr, "ERRO: Falha ao configurar porta serial [%s]. "
              "[ioctl(TCSETAF:%u): %s]\n", dev,
              baud == B1200 ? 1200 :
              baud == B2400 ? 2400 :
              baud == B4800 ? 4800 :
              baud == B9600 ? 9600 : ~0, strerror(errno));
      close(handle);
      handle = -1;
      return -1;
   }

	return handle;
}	/* pin_open_dev() */

#ifdef ANSI
int pin_close_dev(int handle)
#else
int pin_close_dev(handle)
	int handle;
#endif /* ANSI */
{
	return close(handle);
}	/* pin_close_dev() */

/*
 */

#ifdef ANSI
static int pin_send_byte(int handle, char b)
#else
static int pin_send_byte(handle, b)
	int handle; char b;
#endif /* ANSI */
{
	if (write(handle, &b, sizeof(b)) == sizeof(b))
		return 0;
	perror("write()");
	return -1;
}	/* pin_send_byte() */

#ifdef ANSI
static int pin_recv_byte(int handle, char *b)
#else
static int pin_recv_byte(handle, b)
	int handle; char *b;
#endif /* ANSI */
{
	size_t s;
	int n = 0;

	do
	{
		if ((s = read(handle, b, sizeof(*b))) == sizeof(*b))
			return 0;
		if (s)
			perror("read()");
		else if (n++ < 1)
			sleep(1);
	} while ((s == 0) && (n < 2));

	return -1;
}	/* pin_recv_byte() */

#ifdef ANSI
static int pin_dev_flush(int handle)
#else
static int pin_dev_flush(handle)
	int handle;
#endif /* ANSI */
{
	if (ioctl(handle, TCFLSH, 0) == 0)
		return 0;
	perror("ioctl(TCFLSH)");
	return -1;
}	/* pin_dev_flush() */

/* Copied from a "well known" PINPAD "driver" */
#ifdef ANSI
int pin_send(int handle, char *buffer, int buflen, 
		unsigned char proto, unsigned char eot)
#else
int pin_send(handle, buffer, buflen, proto, eot)
	int handle; char *buffer; int buflen; unsigned char proto; unsigned char eot;
#endif /* ANSI */
{
	unsigned char bcc;
	char chC;
	int nI;
	int nTentativas;
	int nRet;
	int nTamTx;

	nTentativas = 3;

	while (nTentativas--) {
		nI = 0;
		nTamTx = buflen;

		/* transmite STX ou SI */

		if (proto == PINPAD_PROTO_STXETX)
			chC = T_STX;
		else
			chC = T_SI;

		if ((nRet = pin_send_byte(handle, chC)) != 0)
			return (-1);

		/* nao faz bcc com o STX */

		/* transmite o buffer e realiza o bcc do buffer */
		bcc = 0;

		while (nTamTx--) {
			chC = buffer[nI++];
			if ((nRet = pin_send_byte(handle, chC)) != 0)
				return (-1);
			bcc ^= chC;
		}

		/* transmite ETX ou SO */

		if (proto == PINPAD_PROTO_STXETX)
			chC = T_ETX;
		else
			chC = T_SO;

		if ((nRet = pin_send_byte(handle, chC)) != 0)
			return (-1);

		bcc ^= chC;

		/* transmite o BCC */
		if ((nRet = pin_send_byte(handle, bcc)) != 0)
			return (-1);

		if (eot != PINPAD_EOTACK_NO)
		{
			/* espera o ACK */
			if ((nRet = pin_recv_byte(handle, &chC)) != 0){
				return (-1);
			}

			if (chC == T_ACK)
			{	/* se ACK OK vai embora... */
				if (eot == PINPAD_EOT_YES){
					/* recebe o EOT final */
					if ((nRet = pin_recv_byte(handle, &chC)) != 0)
						return (-1);

					if (chC != T_EOT)
						return (-1);
				}
				return (0); /* OK */
			}
		}
		else
		{	/* Pin Schlumberger. Command without ACK. */
			return 0; /* OK */
		}
	}

	/* acabaram-se as tentativas */
	return (-1);
}	/* pin_send() */

/* Copied from a "well known" PINPAD "driver" */
#ifdef ANSI
int pin_recv(int handle, char *buffer, int bufsize, unsigned char eot)
#else
int pin_recv(handle, buffer, bufsize, eot)
	int handle; char *buffer; int bufsize; unsigned char eot;
#endif /* ANSI */
{
    int nRet;
    int nI;
    int nTentativas;
    unsigned char chC;
    unsigned char bcc;
	 int nTam;
	 int received;

	 received = 0;

    nTentativas = 3;

    while (nTentativas--) {
		 nTam = bufsize;
	   /* recebe STX ou SI */
	   while (TRUE) {
		  if ((nRet = pin_recv_byte(handle, &chC)) == 0) {
			 if (chC == T_EOT)
				return -1;
			 if (chC == T_STX || chC == T_SI)
				break;
			 if (chC == T_ETX || chC == T_SO) {
				if ((nRet = pin_recv_byte(handle, &chC)) != 0)
				    return -1;
				if ((nRet = pin_send_byte(handle, T_NAK)) != 0)
				    return -1;
				continue;
			 }
		  }
		  else
			 return (-1);
	   }

	   nI = 0;

	   /* nao faz BCC com o STX */
	   bcc = 0;

	   /* recebe o buffer e o ETX e faz o bcc */
	   while (TRUE) {
		  if ((nRet = pin_recv_byte(handle, &chC)) == 0) {
			 bcc ^= chC;
			 if (chC == T_ETX || chC == T_SO)
				break;

			 if (nTam > 0) {
				buffer[nI++] = chC;
			 }
          nTam--;
		  }
		  else
			 return (-1);
	   }

	   received = nI;

	   /* recebe o BCC */
	   if ((nRet = pin_recv_byte(handle, &chC)) != 0)
		  return (-1);

	   /* compara os dois BCCs (o calculado e o recebido...) */

	   if (bcc != chC)                 /* se o BCC nao esta OK */
		  if ((nRet = pin_send_byte(handle, T_NAK)) != 0)
			 return -1;            /* NAK nao foi enviado OK */
		  else
			 continue;               /* NAK enviado OK, volta e tenta de
								 * novo... */

	   else {                          /* se o BCC esta OK */

		  if ((nRet = pin_send_byte(handle, T_ACK)) != 0)
			 return -1;            /* ACK nao foi enviado OK */
		  else
			 break;                  /* se estiver tudo OK... */
	   }
    }

    /* acabaram-se as tentativas... */
    if (nTentativas < 0)
		return (-1);

    if (eot == PINPAD_EOT_YES)
	 {
	   /* recebe o EOT final */
	   if ((nRet = pin_recv_byte(handle, &chC)) != 0)
		  return (-1);

	   if (chC != T_EOT)
		  return (-1);
    }

   if (nTam >= 0)
		return (received);
   else
		return (-1);
}	/* pin_recv() */

/* testpind.c eof ---------------------------------------------------------- */
