/* 30/07/96 13:31
 ******************************************************************************
 **                                                                           *
 **  MODULO : conf.c                                                          *
 **                                                                           *
 **  FUNCAO : tratamento da configuracao                                      *
 **                                                                           *
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  Registro de Alteracoes/Versoes :                                         *
 **  ==============================                                           *
 **                                                                           *
 **  VERSAO   DATA    RESPONSAVEL   DESCRICAO                                 *
 **                                                                           *
 **  V1.00  11/06/96  Magoga        Criacao do arquivo                        *
 **  V1.01  2003/01/24  Tarcisio Kreutz, Perto S/A
 **     - Documentation stated that in the configuration file one or more
 **       blanks or tabs were allowed between any two parameters. But, 
 **       just one blank or tab was actually accepted. Corrected.
 **       Anyway, unless client software gets recompiled with the new client
 **       software the configuration file must be formatted considering the
 **       old 'weakness' (of course, only if server and client use the same 
 **       configuration file).
 **     - Added a new optional parameter: #dtroff. It must be added after the
 **       'speed' parameter (preceeded by blanks or tabs), if the new
 **       behaviour of the 'server' software is desired: Set the DTR of the
 **       serial port to OFF. This allows the software to talk to the new
 **       Pertochek hardware which does just forward the data to a PINPAD
 **       if DTR is ON (when dip switch 4 is ON).
 **       The new parameter has a "comment like" format to be allowed in
 **       configuration files read by old client software (which does not
 **       allow any character different from 'new line' after the last
 **       parameter, unless a comment mark (#) prefixes it).
 **     - Check if the config parameters are correct here. Do not return OK
 **       if bad values are found.
 **                                                                           *
 ******************************************************************************
 *
 ******************************************************************************
 **                                                                           *
 **  COMENTARIOS  :                                                           *
 **                                                                           *
 **                                                                           *
 **  ATENCAO :                                                                *
 **  =======                                                                  *
 **                                                                           *
 **                                                                           *
 ******************************************************************************
 */

/**#######################  I N C L U D E S  #################################*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termio.h>

#include "perto.h"

/*#######################  C O N S T A N T E S  ##############################*/


/*#######################  P R O T O T I P O S  ##############################*/

#ifdef ANSI
static int le_linha(FILE *, st_serverinfo_t *, int *);
#endif

/*#######################  V A R I A V E I S  ################################*/

/*=======================  L o c a i s  ======================================*/


/*=======================  P u b l i c a s  ==================================*/

/*=======================  E x t e r n a s  ==================================*/

/*#######################  R O T I N A S  ####################################*/

/* Load configuration of all installed Pertochek */
#ifdef ANSI
int inic_conf(st_config_t * p_conf, char *arq)
#else
int inic_conf(p_conf, arq)
     st_config_t *p_conf;
     char *arq;
#endif
{
   int fim = 0;
   int i;
   FILE *h_arq;

   p_conf->num_imp = 0;

   if ((h_arq = fopen(arq, "r")) == NULL)
   {
      return (FALSE);
   }

   for (i = 0; i < MAX_PERTO; i++)
   {
      if (le_linha(h_arq, &p_conf->info[i], &fim) == FALSE)
      {
         fclose(h_arq);
         return (FALSE);
      }

      if (fim)
         break;

      p_conf->num_imp++;
   }

   fclose(h_arq);
   return (TRUE);
}

/* Load configuration of a specific installed Pertochek */
#ifdef ANSI
int pcheck_get_config(char *nome, char *arq, st_serverinfo_t *info, int *err)
#else
int pcheck_get_config(nome, arq, info, err)
	char *nome;
	char *arq;
	st_serverinfo_t *info;
	int *err;
#endif
{
   int fim = 0;
   int i;
   FILE *h_arq;

   if ((h_arq = fopen(arq, "r")) == NULL)
   {
		*err = P_ERRO_CONFIG;
      return (FALSE);
   }

	/* NOTE: We use MAX_PERTO here so that client & server will agree about
	 * the number of valid config entries thay consider... */
   for (i = 0; i < MAX_PERTO; i++)
   {
      if (le_linha(h_arq, info, &fim) == FALSE)
			break;
      if (fim)
         break;

      if (strcmp(info->name, nome) == 0)
      {
         fclose(h_arq);
         return (TRUE);
      }
   }
	if ((i >= MAX_PERTO) || fim)
		*err = P_ERRO_NOME_PERTO;
	else
		*err = P_ERRO_CONFIG;

   fclose(h_arq);
   return (FALSE); /* NOT FOUND or ERROR */
}	/* pcheck_get_config() */

/* Translate one config record from configuration file.
 * Returns: TRUE: Input translated (or EOF, in which case '*fim' is set to TRUE)
 *          FALSE: Error.
 * LINHA:
 *  #: Se iniciar com '#', considerada comentario e ignorada.
 *  1. nome da impressora - na primeira posicao
 *  2. 1 ou mais brancos ou tab
 *  3. nome da porta serial
 *  4. 1 ou mais brancos ou tab
 *  5. modelo da impressora
 *  6. 1 ou mais brancos ou tab
 *  7. velocidade
 *  Obs1.: caractere '#' indica fim de linha (comentario), exceto 
 *         quando for '#dtroff', que eh um parametro:
 *  8. #dtroff
 *     Indica que o sinal DTR da serial deve ser desabilitado.
 *  O restante da linha (ateh 'new line') serah ignorado.
 *  O tamanho maximo da linha eh de 500 bytes (de fato, um pouco mais!)
 */
#ifdef ANSI
static int le_linha(FILE * h_arq, st_serverinfo_t * i_ptr, int *fim)
#else
int le_linha(h_arq, i_ptr, fim)
     FILE *h_arq;
     st_serverinfo_t *i_ptr;
     int *fim;
#endif
{
   int n;
   char record[512];
   /* NOTE: We use larger arrays for sscanf() not to cause trouble if the
    *       user sets parameters a "bit" larger than expected. */
   char name[81];
   char msgkey[81];
   char device[81];
   char model[81];
   char speed[81];
   char dtroff[81];
   char trash[81];
   /* --------------- */

   /* Read at least one valid record from the config file (unless error...) */
   while (fgets(record, sizeof(record), h_arq))
   {
      if (((n = sscanf(record, "%s %s %s %s %s %s %s",
                       name, msgkey, device, model, speed, dtroff, trash)) >= 5)
          && (name[0] != '#' /* Comment? */ ))
      {
         strncpy(i_ptr->name, name, sizeof(i_ptr->name));
         i_ptr->name[sizeof(i_ptr->name) - 1] = 0;

         strncpy(i_ptr->device, device, sizeof(i_ptr->device));
         i_ptr->device[sizeof(i_ptr->device) - 1] = 0;

         i_ptr->msgkey = atol(msgkey); /* TODO: Should check if numeric. */

         i_ptr->pchekcaps = PCHEK_CAPS_NONE;
         if (strcmp(model, "401C") == 0 || strcmp(model, "402C") == 0)
            i_ptr->pchekcaps |= PCHEK_CAPS_POSMARK;
         if ((n > 5) && (!strcmp(dtroff, "#dtroff")))
            i_ptr->pchekcaps |= PCHEK_CAPS_DTROFF;
         switch (atoi(speed))
         {
         case 1200:
            i_ptr->baud = B1200;
            break;
         case 2400:
            i_ptr->baud = B2400;
            break;
         case 4800:
            i_ptr->baud = B4800;
            break;
         case 9600:
            i_ptr->baud = B9600;
            break;
         default:
#ifndef CLIENT_NO_MESSAGES
            fprintf(stderr, "ERRO: Configuracao velocidade incorreta [%s]\n",
                    speed);
#endif /* CLIENT_NO_MESSAGES */
            return (FALSE);
         }

         return (TRUE);         /* Record translated */
      }
   }

   if (feof(h_arq))
   {
      *fim = TRUE;
      return (TRUE);            /* EOF */
   }

#ifndef CLIENT_NO_MESSAGES
   fprintf(stderr, "ERRO: Falha leitura arquivo configuracao\n");
#endif /* CLIENT_NO_MESSAGES */

   return (FALSE);              /* ERROR */
}                               /* le_linha() */

/* conf.c eof -------------------------------------------------------------- */
